/******************************************************************************

Copyright 2003, 2004 Christian Gollan

This file is part of the W2D package.

The W2D package is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

The W2D package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with the W2D package; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

******************************************************************************/

#include <stdio.h>
#include <argp.h>
#include <math.h>
#include <string.h>
#include "DIS.h"
#include "config.h"

const char *argp_program_version =
VERSION;

const char *argp_program_bug_address =
PACKAGE_BUGREPORT;

/* This structure is used by main to communicate with parse_opt. */
struct arguments
{
  char *disdat_name;            /* ARG1 */
  unsigned int mode;
  int k;
  int disfield_nr;
  char* alpha;
  double variance;
};

/*
   OPTIONS.  Field 1 in ARGP.
   Order of fields: {NAME, KEY, ARG, FLAGS, DOC}.
*/
static struct argp_option options[] =
{
  {"calc", 'c', 0, 0, "Calculate average and variance for each class"},
  {"verbose", 'v', 0, 0, "Verbose output"},
  {"mode",'M',"kNN|KD",0,"Choose the classifier"},
  {"matrix",'m',0 ,0 ,"Classifiedmatrix"},
  {"disfield",'f',"N",0,"Select the N'th disfield, counted from 0"},
  {"alpha", 'a', "R:R:R:...", 0, "Scalefactor for variance"},
  {"variance", 'V', "R", 0, "Variancefactor"},
  {0,'k',"N",0,"Select the N best distances for kNN, default 1"},
  {0}
};


/*
   PARSER. Field 2 in ARGP.
   Order of parameters: KEY, ARG, STATE.
*/
static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  struct arguments *arguments = state->input;

  switch (key)
    {
    case 'v':
      arguments->mode |= DIS_verbose_out;
      break;
    case 'm':
      arguments->mode |= DIS_verbose_matrix;
      break;
    case 'c':
      arguments->mode |= DIS_verbose_calc;
      break;
    case 'M':
      arguments->mode&=65536-255;
      if(!strcmp(arg,"kNN"))
	arguments->mode|=DIS_class_kNN;
      else
	if(!strcmp(arg,"KD"))
	  arguments->mode|=DIS_class_KD;	
	else
	  argp_usage(state);
      break;
    case 'f':
      if(sscanf(arg,"%d%s",&(arguments->disfield_nr),arg)!=1
	 || arguments->disfield_nr<0)
	argp_usage(state); 
      break;
    case 'k':
      if(sscanf(arg,"%d%s",&(arguments->k),arg)!=1
	 || arguments->k<1)
	argp_usage(state); 
      break;
    case 'V':
      if(sscanf(arg,"%lf%s",&(arguments->variance),arg)!=1
	 || arguments->variance<=0)
	argp_usage(state); 
      break;
    case 'a':
      arguments->alpha=arg;
      break;
    case ARGP_KEY_ARG:
      if (state->arg_num >= 1)
	{
	  argp_usage(state);
	}
      arguments->disdat_name = arg;
      break;
    case ARGP_KEY_END:
      if (state->arg_num < 1)
	{
	  argp_usage (state);
	}
      break;
    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}

/*
   ARGS_DOC. Field 3 in ARGP.
   A description of the non-option command-line arguments
     that we accept.
*/
static char args_doc[] = "FILE";

/*
  DOC.  Field 4 in ARGP.
  Program documentation.
*/
static char doc[] =
"classifier -- The program reads disdatfile FILE with squared distances and print the errorrate for the chosen classifier.\vWhen FILE is -, the program reads standard input.";

/*
   The ARGP structure itself.
*/
static struct argp argp = {options, parse_opt, args_doc, doc};


int main (int argc, char **argv)
{
  struct arguments arguments;
  FILE *outstream;
  DISDAT disdat;

  /* Set argument defaults */
  arguments.mode = DIS_class_kNN;
  arguments.k = 1;
  arguments.disfield_nr = -1;
  arguments.disdat_name = "";
  arguments.alpha = "";
  arguments.variance=1.0;

  argp_parse (&argp, argc, argv, 0, 0, &arguments);
  setbuf (stdout, NULL);

  disdat=DIS_read_disdat_file(arguments.disdat_name);
  disdat.k=arguments.k;
  disdat.classifier_type=arguments.mode;
  disdat.disfield_nr = arguments.disfield_nr;

  DIS_disdat_set_variance(&disdat,arguments.alpha, arguments.variance);

  if(disdat.classifier_type & DIS_verbose_calc)
    {
      DIS_disdat_calc(disdat);
      DIS_disdat_print_best_ER(disdat);
    }
  else
    {
      DIS_disdat_classifier(disdat);
    }
  // disdat_print(disdat);
  DIS_disdat_free(disdat);

  return 0;
}
