/******************************************************************************

Copyright 2003, 2004 Christian Gollan

This file is part of the W2D package.

The W2D package is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

The W2D package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with the W2D package; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

******************************************************************************/

#include <stdio.h>
#include <argp.h>
#include <stdlib.h>
#include <float.h>
#include <math.h>
#include "IMG.h"
#include "config.h"

const char *argp_program_version =
VERSION;

const char *argp_program_bug_address =
PACKAGE_BUGREPORT;

/* This structure is used by main to communicate with parse_opt. */
struct arguments
{
  char* etlm_filename;
  char* jf_filename;
  int verbose;              /* The -v flag */
  int downscale;
};

/*
   OPTIONS.  Field 1 in ARGP.
   Order of fields: {NAME, KEY, ARG, FLAGS, DOC}.
*/
static struct argp_option options[] =
{
  {"verbose", 'v', 0, 0, "Verbose output"},
  {"scale", 'd', 0, 0, "Scale images to 16x16 pixel"},
  {0}
};


/*
   PARSER. Field 2 in ARGP.
   Order of parameters: KEY, ARG, STATE.
*/
static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  struct arguments *arguments = state->input;

  switch (key)
    {
    case 'v':
      arguments->verbose = 1;
      break;
    case 'd':
      arguments->downscale = 1;
      break;    
    case ARGP_KEY_ARG:
      if (state->arg_num >= 2)
	{
	  argp_usage(state);
	}
      switch(state->arg_num)
	{
	case 0:
	  arguments->etlm_filename = arg;
	  break;
	case 1:
	  arguments->jf_filename = arg;
	  break;
	}
      break;
    case ARGP_KEY_END:
      if (state->arg_num < 2)
	{
	  argp_usage (state);
	}
      break;
    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}

/*
   ARGS_DOC. Field 3 in ARGP.
   A description of the non-option command-line arguments
     that we accept.
*/
static char args_doc[] = "[FILE1] [FILE2]";

/*
  DOC.  Field 4 in ARGP.
  Program documentation.
*/
static char doc[] =
"etlm2jf -- A programm to convert ETLM files.\nFILE1 is the ETLM data file and FILE2 will be created in jf-format.";

/*
   The ARGP structure itself.
*/
static struct argp argp = {options, parse_opt, args_doc, doc};


IMG_list*
IMG_etlm_to_img_list(char* filename)
{
  FILE* file;
  int total_img_nr;
  IMG_list* img_list;
  int img_nr;
  int data_pos;
  unsigned char* buffer;

  if ((file = fopen (filename, "r")) == NULL)
    {
      fprintf (stderr, "can't open file %s\n", filename);
      exit (0);
    }

  buffer=(unsigned char*) malloc(4032*sizeof(unsigned char));
  if(buffer==NULL)
    {
      fprintf (stderr, "not enough memory for image_buffer in etlm_to_img_list\n");
      exit (0); 
    } 
  
  total_img_nr=0;
  while (fread(buffer,sizeof(unsigned char),2052,file)==2052)
    total_img_nr++;

  rewind(file);

  img_list=IMG_create_new_list(total_img_nr,2);

  for(img_nr=0;img_nr<total_img_nr;img_nr++)
    {
      if(fread(buffer,sizeof(unsigned char),2052,file)
	 !=2052)
	{
	  fprintf (stderr, "unexpected end of file %s\n", filename);
	  exit (0); 
	}
      img_list->img[img_nr].data_list[0]
	=(double*)malloc(4032*sizeof(double));
      if(img_list->img[img_nr].data_list[0]==NULL)
	{
	  fprintf (stderr, "not enough memory for img_list in etlm_to_img_list\n");
	  exit (0); 
	} 
      for(data_pos=0;data_pos<2016;data_pos++)
	{
	  img_list->img[img_nr].data_list[0][data_pos*2]=
	    (double)(buffer[data_pos+32]>>4);
	  img_list->img[img_nr].data_list[0][(data_pos*2)+1]=
	    (15 & buffer[data_pos+32]);
	}
      img_list->img[img_nr].data_length[0]=4032;
      img_list->img[img_nr].x_size=64;
      img_list->img[img_nr].y_size=63;
      img_list->img[img_nr].class=(int) (buffer[24]*256) + buffer[25] ;
	/*( (buffer[12]*16777216) + (buffer[13]*65536)
	  + (buffer[14]*256) + buffer[15] );      */
    }
  free(buffer);

  fclose (file);
  return(img_list);
}

int
IMG_ETLM_to_jf (unsigned char* buffer,FILE* jf_file, int downscale)
{
  int nr,class;
  int i,j,x,y;
  unsigned char img_data[4096];
  unsigned char temp;

  nr = (int) (buffer[0]*256 + buffer[1]);
  class = (int) ( (buffer[12]*16777216) + (buffer[13]*65536)
		  + (buffer[14]*256) + buffer[15] ); 
  //  printf("character:%c%c nr:%d class:%d\n",buffer[2],buffer[3],nr,class);
  fprintf(jf_file,"%d",(int) (buffer[2]*256 + buffer[3]));
  
  for(i=0;i<2016;i++)
    {
      img_data[i*2]=(buffer[i+32]>>4);
      img_data[(i*2)+1]=(15 & buffer[i+32]);
    }
  for(i=4096;i>=4032;i--)
    img_data[i]=0;
  
  if(downscale)
    {
      for(i=0;i<4096;i+=256)
	for(j=0;j<64;j+=4)
	  {
	    temp=0;
	    for(y=0;y<256;y+=64)
	      for(x=0;x<4;x++)
		temp+=img_data[i+j+y+x];
	    fprintf(jf_file," %d", temp);
	  }
    } 
  else
    {
      for(i=0;i<4096;i++)
	fprintf(jf_file," %d", img_data[i]);
    }
  fprintf(jf_file,"\n");
  return (1);
}

int IMG_ETLM_list_to_jf (char* ETL_name,char* jf_name, int downscale)
{
  FILE* ETL_file;
  FILE* jf_file;
  unsigned char buffer[2052];
  int total_img_nr=0;

  if ((ETL_file = fopen (ETL_name, "r")) == NULL)
    {
      fprintf (stderr, "can't open file %s\n", ETL_name);
      exit (0);
    }
  if (jf_name != NULL)
    {
      if ((jf_file = fopen (jf_name, "w")) == NULL)
	{
	  fprintf (stderr, "can't create file %s\n", jf_name);
	  exit (0);
	}
    }
  else
    jf_file=stdout;

  if(downscale)
    fprintf (jf_file, "65536 256\n");
  else
    fprintf (jf_file, "65536 4096\n");

  while (read (fileno(ETL_file),buffer, 2052)==2052)
    {
      IMG_ETLM_to_jf(buffer,jf_file,downscale);
      total_img_nr++;
    }

  fclose (ETL_file);
  if(jf_file!=stdout)
    fclose (jf_file);
  return(total_img_nr);
}

int
main (int argc, char *argv[])
{
  struct arguments arguments;
  IMG_list* img_list;

  /* Set argument defaults */
  arguments.downscale = 0;
  arguments.verbose = 0;

  argp_parse (&argp, argc, argv, 0, 0, &arguments);

  img_list = IMG_etlm_to_img_list(arguments.etlm_filename);
  
 if(arguments.verbose)   
    fprintf(stderr,"Total of %d images\n",img_list->total_nr);

 IMG_save_list(img_list,arguments.jf_filename);
  IMG_free_list(img_list);

  exit (1);
}
