/******************************************************************************

Copyright 2003, 2004 Christian Gollan

This file is part of the W2D package.

The W2D package is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

The W2D package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with the W2D package; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

******************************************************************************/

#include <stdio.h>
#include <argp.h>
#include <float.h>
#include <math.h>
#include "IMG.h"
#include "Xdraw.h"
#include "config.h"

const char *argp_program_version =
VERSION;

const char *argp_program_bug_address =
PACKAGE_BUGREPORT;

/* This structure is used by main to communicate with parse_opt. */
struct arguments
{
  char* jf_file_name;            /* ARG1 */
  char* png_file_name;
  int img_start_nr;
  int img_total_nr;
  int verbose;
  int inverse;
  int class;
};

int argp_init_arg(struct arguments* arguments)
{
  arguments->png_file_name="";
  arguments->img_start_nr=0;
  arguments->img_total_nr=-1;
  arguments->verbose=0;
  arguments->inverse=0;
  arguments->class=-1;
}

/*
   OPTIONS.  Field 1 in ARGP.
   Order of fields: {NAME, KEY, ARG, FLAGS, DOC}.
*/
static struct argp_option options[] =
{
  {"outname",'O',"[FILE...",0,"Names of PNG files start with"},
  {"start", 'S',"N", 0, "Image number to start"}, 
  {"total", 'E', "N", 0, "Total number of images to create"},
  {"verbose", 'v', 0, 0, "Verbose output"}, 
  {"inverse", 'i', 0, 0, "Inverse image output"},
  {"class", 'C',"N",0, "Class to seperate"}, 
  {0}
};


/*
   PARSER. Field 2 in ARGP.
   Order of parameters: KEY, ARG, STATE.
*/
static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  struct arguments *arguments = state->input;

  switch (key)
    {
    case 'O':
      arguments->png_file_name=arg;
      break;
    case 'S':
      if(sscanf(arg,"%d%s",&(arguments->img_start_nr),arg)!=1
	 || arguments->img_start_nr<0)
	argp_usage(state); 
      break;
    case 'E':
      if(sscanf(arg,"%d%s",&(arguments->img_total_nr),arg)!=1
	 || arguments->img_total_nr<1)
	argp_usage(state); 
      break;
   case 'C':
      if (sscanf (arg, "%d%s", &(arguments->class), arg) != 1 || arguments->class<0)
	  argp_usage(state);
      break;
    case 'i':
      arguments->inverse=1;
      break;
    case 'v':
      arguments->verbose=1;
      break;
    case ARGP_KEY_ARG:
      if (state->arg_num >= 1)
	{
	  argp_usage(state);
	}
      arguments->jf_file_name = arg;
      break;
    case ARGP_KEY_END:
      if (state->arg_num < 1)
	{
	  argp_usage (state);
	}
      break;
    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}

/*
   ARGS_DOC. Field 3 in ARGP.
   A description of the non-option command-line arguments
     that we accept.
*/
static char args_doc[] = "[FILE]";

/*
  DOC.  Field 4 in ARGP.
  Program documentation.
*/
static char doc[] =
"jf2png -- The program reads FILE, and writes PNG images.";

/*
   The ARGP structure itself.
*/
static struct argp argp = {options, parse_opt, args_doc, doc};


#define MAX_SAVE_NAME_LENGTH 200
int
main (int argc, char *argv[])
{
  struct arguments arguments;
  IMG_list* img_list;
  int img_nr;
  char buffer[MAX_SAVE_NAME_LENGTH];
  int img_depth;
  double min,max;

  /* Set argument defaults */
  argp_init_arg(&arguments);

  argp_parse (&argp, argc, argv, 0, 0, &arguments);

  img_list=IMG_load_list(arguments.jf_file_name);

  if(arguments.img_total_nr==-1)
    arguments.img_total_nr=img_list->total_nr;


  min = DBL_MAX;
  max = -DBL_MAX;
  for(img_depth=0;img_depth<=IMG_get_depth_list(img_list);img_depth++)
    IMG_get_min_max_list (img_list, img_depth, &min, &max);  
      
  if(arguments.inverse!=0)
    for(img_depth=0;img_depth<=IMG_get_depth_list(img_list);img_depth++)
      IMG_norm_list (img_list, img_depth, min, max, 1.0, 0.0);
  else
    for(img_depth=0;img_depth<=IMG_get_depth_list(img_list);img_depth++)
      IMG_norm_list (img_list, img_depth, min, max, 0.0, 1.0);


  for(img_nr=arguments.img_start_nr;
      ((img_nr<arguments.img_start_nr+arguments.img_total_nr)
       && (img_nr<img_list->total_nr));img_nr++)
    {
      if(strlen(arguments.png_file_name)+strlen(img_list->img[img_nr].name)+4>=MAX_SAVE_NAME_LENGTH)
	{
	  fprintf(stderr,"png_file_name length exceed limit %d\n", MAX_SAVE_NAME_LENGTH);
	  exit(-1);
	}
      sprintf(buffer,"%s%s.png",arguments.png_file_name,img_list->img[img_nr].name);
      if(arguments.class==-1 || arguments.class==img_list->img[img_nr].class)
	{
	  if(arguments.verbose!=0)
	    fprintf(stdout,"#save image number %d of class %d with filename: %s\n",
		    img_nr, img_list->img[img_nr].class, buffer);
	  IMG_GD_save_img_png(img_list->img[img_nr], buffer);
	}
      else
	arguments.img_total_nr++;

    }
  
  IMG_free_list(img_list);

  exit (1);
}
