/******************************************************************************

Copyright 2003, 2004 Christian Gollan

This file is part of the W2D package.

The W2D package is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

The W2D package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with the W2D package; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <argp.h>
#include <string.h>
#include <math.h>
#include "IMG.h"
#include "config.h"

#define FILTER_warparound 1

const char *argp_program_version =
VERSION;

const char *argp_program_bug_address =
PACKAGE_BUGREPORT;

/* This structure is used by main to communicate with parse_opt. */
struct arguments
{
  char* args[2];            /* ARG1 */
  char* filter_name;
  int filter_mode;
  int verbose;
  char* filterfilename;
};

/*
   OPTIONS.  Field 1 in ARGP.
   Order of fields: {NAME, KEY, ARG, FLAGS, DOC}.
*/
static struct argp_option options[] =
{
  {"filter",'F',"<name>",0,"Choose the filter"},
  {"warparound", 'w', 0, 0, "Warparound"}, 
  {"filterfile", 'I', "[FILE]", 0, "Load filterfile [FILE]"},
  {"verbose", 'v', 0, 0, "Verbose output"}, 
  {0}
};


/*
   PARSER. Field 2 in ARGP.
   Order of parameters: KEY, ARG, STATE.
*/
static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  struct arguments *arguments = state->input;

  switch (key)
    {
    case 'F':
      arguments->filter_name=arg;
      break;
    case 'I':
      arguments->filterfilename=arg;
      break;
    case 'w':
      arguments->filter_mode|= FILTER_warparound;
      break;
    case 'v':
      arguments->verbose=1;
      break;
    case ARGP_KEY_ARG:
      if (state->arg_num >= 2)
	{
	  argp_usage(state);
	}
      arguments->args[state->arg_num] = arg;
      break;
    case ARGP_KEY_END:
      if (state->arg_num != 2)
	{
	  argp_usage (state);
	}
      break;
    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}

/*
   ARGS_DOC. Field 3 in ARGP.
   A description of the non-option command-line arguments
     that we accept.
*/
static char args_doc[] = "[FILE1] [FILE2]";

/*
  DOC.  Field 4 in ARGP.
  Program documentation.
*/
static char doc[] =
"jffilter -- The program reads FILE1, filters the images and write FILE2.\vThe default filterfile is \"filterlist.fildat\"\nExample:\n\tfilterA 2x2\n\t+0 -1\n\t-1 +0\n\n\tfilterB 3x1\n\t0 -1.0 0\n";

/*
   The ARGP structure itself.
*/
static struct argp argp = {options, parse_opt, args_doc, doc};

#define BUFFER_LENGTH 100
IMG_filter* img_load_filter_list(char* filename)
{
  FILE* filterfile;
  char buffer[BUFFER_LENGTH];
  char temp_string[BUFFER_LENGTH];
  int line_nr=0;
  int filter_x_size;
  int filter_y_size;
  int x,y;
  double temp_double;
  IMG_filter* filter_list;
  int filter_nr;
  int filter_list_length;

  filter_list_length=10;
  filter_list=(IMG_filter*) malloc(sizeof(IMG_filter)*filter_list_length);
  if(filter_list==NULL)
    {
      fprintf (stderr, "not enough memory for filter_list\n");
      exit (0);
    }

  if((filterfile = fopen (filename, "r")) == NULL)
    {
      fprintf (stderr, "can't open file %s\n", filename);
      exit (0);
    }
  fprintf(stdout,"#Read filter_file: %s\n",filename);
  filter_nr=0;
  while(buffer==fgets(buffer, BUFFER_LENGTH, filterfile))
    {
      line_nr++;
      if(strlen(buffer)>BUFFER_LENGTH-2)
      {
        fprintf (stderr, "line %d exceed max. LINE_LENGTH %d\n",line_nr,BUFFER_LENGTH-2);
	fclose (filterfile);
        exit (0);
      }  
      if(sscanf(buffer,"%s %dx%d",temp_string,&filter_x_size,&filter_y_size)==3)
	{
	  if(filter_nr+2>filter_list_length)
	    {
	      filter_list_length*=2;
	      filter_list=
		(IMG_filter*)realloc(filter_list,sizeof(IMG_filter)*filter_list_length);
	      if(filter_list==NULL)
		{
		  fprintf (stderr, "not enough memory for filter_list\n");
		  exit (0);
		}	
	    }
	  if(filter_x_size==0 || filter_y_size==0)
	    {
	      fprintf (stderr, "can not create %dx%d filter %s\n",
		       filter_x_size,filter_y_size,
		       temp_string);
	      exit (0);
	    }	
	  
	  filter_list[filter_nr].name=
	    (char*) malloc(sizeof(char)*(strlen(temp_string)+1));
	  if(filter_list[filter_nr].name==NULL)
	    {
	      fprintf (stderr, "not enough memory for filter_list\n");
	      exit (0);
	    }	
	  strcpy(filter_list[filter_nr].name,temp_string);
	  filter_list[filter_nr].x_size=filter_x_size;
	  filter_list[filter_nr].y_size=filter_y_size;
	  filter_list[filter_nr].data=
	    (double*)malloc(sizeof(double)*filter_x_size*filter_y_size);
	  if(filter_list[filter_nr].data==NULL)
	    {
	      fprintf (stderr, "not enough memory for filter_list\n");
	      exit (0);
	    }	
	  
	  for(y=0;y<filter_y_size;y++)
	    {
	      for(x=0;x<filter_x_size;x++)
		{
		  fscanf(filterfile,"%lf",&temp_double);
		  filter_list[filter_nr].data[x+(y*filter_x_size)]=
		    temp_double;
		}
	    }
	  fprintf(stdout,"#\tfound %dx%d filter: %s\n",
		  filter_x_size,filter_y_size,
		  filter_list[filter_nr].name);

	  filter_nr++;
	}
    }
  filter_list[filter_nr].name=NULL;
  filter_list[filter_nr].data=NULL;
  filter_list[filter_nr].x_size=0;
  filter_list[filter_nr].y_size=0; 
  fclose (filterfile);
  return(filter_list);
}

IMG_filter img_get_filter(IMG_filter* filter_list,char* filter_name)
{
  int filter_nr=0;
  
  while(filter_list[filter_nr].data!=NULL)
    {
      if(!strcmp(filter_name,filter_list[filter_nr].name))
	break;
      filter_nr++;
    }
  
  
  return(filter_list[filter_nr]);
}

int img_free_filter_list(IMG_filter* filter_list)
{
  int filter_nr=0;
  
  while(filter_list[filter_nr].data!=NULL)
    {
      free(filter_list[filter_nr].data);
      free(filter_list[filter_nr].name);
      filter_nr++;
    }
  free(filter_list);
  return(1);
}

int
img_filter_img (IMG * img, IMG_filter filter)
{
  int x, y, f_x, f_y, f_ox, f_oy, x_temp, y_temp;
  double f_temp;
  double *temp_data;
  double f_div;
  int img_depth;
  temp_data = (double*) malloc (sizeof (double) * img->x_size * img->y_size);
  if (temp_data == NULL)
    {
      fprintf (stderr, "not enough memory for IMG_filter_img\n");
      exit (0);
    }

  f_ox = -(filter.x_size - 1) / 2;
  f_oy = -(filter.y_size - 1) / 2;
  
  
  for(img_depth=0;img->dis_mode[img_depth]!=-2;img_depth++)
    {
      if(img->data_length[img_depth]==img->x_size*img->y_size)
	{
	  for (y = 0; y < img->y_size; y++)
	    for (x = 0; x < img->x_size; x++)
	      {
		f_div = 1;
		f_temp = 0;
		for (f_y = 0; f_y < filter.y_size; f_y++)
		  for (f_x = 0; f_x < filter.x_size; f_x++)
		    {
		      x_temp = f_x + f_ox + x;
		      y_temp = f_y + f_oy + y;
		      switch(filter.mode)
			{
			case 0:
			  if (x_temp >= 0 && x_temp < img->x_size && y_temp >= 0
			      && y_temp < img->y_size)
			    f_temp +=
			      filter.data[(f_y * filter.x_size) + f_x]
			      * img->data_list[img_depth][(y_temp * img->x_size) +
							  x_temp];
			  break;
			case FILTER_warparound:
			  f_temp +=
			    filter.data[(f_y * filter.x_size) + f_x]
			    * img->data_list[img_depth][(((y_temp+img->y_size)%img->y_size)
							 * img->x_size) +
							((x_temp+img->x_size)%img->x_size)];  
			  break;
			}
		    }
		temp_data[(y * img->x_size) + x] = f_temp / f_div;
	      }
	  
	  for (x = 0; x < img->x_size * img->y_size; x++)
	    img->data_list[img_depth][x] = temp_data[x];
	}
    }
  free (temp_data);
  return 0;
}

int
img_filter_list (IMG_list * img_list, IMG_filter filter)
{
  int img_nr;

  for (img_nr = 0; img_nr < img_list->total_nr; img_nr++)
    img_filter_img (&(img_list->img[img_nr]), filter);

  fprintf (stdout, "$IMG_filter_list(%s,%s);\n",
	   img_list->name,filter.name);

  return (0);
}


int main (int argc, char **argv)
{
  struct arguments arguments;
  IMG_list* img_list;
  IMG_filter* filter_list;
  IMG_filter filter;
  /* Set argument defaults */
  arguments.filter_name = "";
  arguments.filterfilename = "filterlist.fildat";
  arguments.filter_mode = 0;
  arguments.verbose = 0;
  arguments.args[0] = "";
  arguments.args[1] = "";

  argp_parse (&argp, argc, argv, 0, 0, &arguments);
  filter_list=img_load_filter_list(arguments.filterfilename);
  filter=img_get_filter(filter_list,arguments.filter_name);
  filter.mode=arguments.filter_mode;
  if(filter.data!=NULL)
    {
      fprintf(stdout,"#Selected filter:%s with mode:%d (0=normal/%d=warparound)\n",
	      filter.name,filter.mode,FILTER_warparound);
      img_list=IMG_load_list (arguments.args[0]);
      img_filter_list(img_list,filter);
      IMG_save_list(img_list,arguments.args[1]);
      IMG_free_list(img_list);
    }
  else
    {
      if(!strcmp(arguments.filter_name,""))
	fprintf(stderr,"No filter selected\n");
      else
	fprintf(stderr,"Can not find filter %s\n", arguments.filter_name);
    }
  img_free_filter_list(filter_list);
  return 0;
}
