/******************************************************************************

Copyright 2003, 2004 Christian Gollan

This file is part of the W2D package.

The W2D package is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

The W2D package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with the W2D package; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <argp.h>
#include "IMG.h"
#include "config.h"

#define IMG_PAD_same_size 1
#define IMG_PAD_quadratic 2

const char *argp_program_version =
VERSION;

const char *argp_program_bug_address =
PACKAGE_BUGREPORT;

/* This structure is used by main to communicate with parse_opt. */
struct arguments
{
  char *args[2];            /* ARG1 */
  double padding_value;
  int mode;
  int border;
};

/*
   OPTIONS.  Field 1 in ARGP.
   Order of fields: {NAME, KEY, ARG, FLAGS, DOC}.
*/
static struct argp_option options[] =
{
  {"value", 'p', "R", 0, "Paddingvalue for added pixels"}, 
  {"quadratic",'q', 0, 0, "Pad to quadratic images"},
  {"samesize",'s', 0, 0, "Pad each image to same size"},
  {"border",'b',"N",0, "Add border to images"},
  {0}
};


/*
   PARSER. Field 2 in ARGP.
   Order of parameters: KEY, ARG, STATE.
*/
static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  struct arguments *arguments = state->input;

  switch (key)
    {
    case 'p':
      if(sscanf(arg,"%lf%s",&(arguments->padding_value),arg)!=1)
	argp_usage(state); 
      break;
    case 'b':
      if(sscanf(arg,"%d%s",&(arguments->border),arg)!=1)
	argp_usage(state); 
      break;    
    case 'q':
      arguments->mode+=IMG_PAD_quadratic;
      break;
    case 's':
      arguments->mode+=IMG_PAD_same_size;
      break;
    case ARGP_KEY_ARG:
      if (state->arg_num >= 2)
	{
	  argp_usage(state);
	}
      arguments->args[state->arg_num] = arg;
      break;
    case ARGP_KEY_END:
      if (state->arg_num < 2)
	{
	  argp_usage (state);
	}
      break;
    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}

/*
   ARGS_DOC. Field 3 in ARGP.
   A description of the non-option command-line arguments
     that we accept.
*/
static char args_doc[] = "[FILE1] [FILE2]";

/*
  DOC.  Field 4 in ARGP.
  Program documentation.
*/
static char doc[] =
"jfpad -- The program reads FILE1, and pads the images to write FILE2.";

/*
   The ARGP structure itself.
*/
static struct argp argp = {options, parse_opt, args_doc, doc};

int img_pad_list(IMG_list* img_list,double padding_value,int mode)
{
  int img_nr;
  int max_x;
  int max_y;
  int min_x;
  int min_y;


  max_x=IMG_get_max_img_x_size_list(img_list);
  max_y=IMG_get_max_img_y_size_list(img_list);
  min_x=IMG_get_min_img_x_size_list(img_list);
  min_y=IMG_get_min_img_y_size_list(img_list);

  printf("#max_x_size:%d max_y_size:%d min_x_size:%d min_y_size:%d\n",
	 max_x,max_y,min_x,min_y);
 
  if(mode&IMG_PAD_same_size)
    {
      if(mode&IMG_PAD_quadratic){
	if(max_x>max_y){
	  max_y=max_x;}
	else{
	  max_x=max_y;}
      }
      printf("#Pad all images to x_size:%d y_size:%d with value:%f\n",max_x,max_y,
	     padding_value);
      for(img_nr=0;img_nr<img_list->total_nr;img_nr++)
	{	  
	  IMG_add_border (&(img_list->img[img_nr]),
			  (max_x-img_list->img[img_nr].x_size+1)/2,
			  (max_x-img_list->img[img_nr].x_size)/2,
			  (max_y-img_list->img[img_nr].y_size+1)/2,
			  (max_y-img_list->img[img_nr].y_size)/2,
			  padding_value);
	}
    }
  else
    {
      if(mode & IMG_PAD_quadratic)
	{
	  printf("#Pad all images to quadratic image size with value:%f\n",
		 padding_value);
	  for(img_nr=0;img_nr<img_list->total_nr;img_nr++)
	    {
	      if(img_list->img[img_nr].x_size>img_list->img[img_nr].y_size)
		{
		  max_x=img_list->img[img_nr].x_size;
		  max_y=img_list->img[img_nr].x_size;
		}
	      else
		{
		  max_x=img_list->img[img_nr].y_size;
		  max_y=img_list->img[img_nr].y_size;
		}
	      IMG_add_border (&(img_list->img[img_nr]),
			      (max_x-img_list->img[img_nr].x_size+1)/2,
			      (max_x-img_list->img[img_nr].x_size)/2,
			      (max_y-img_list->img[img_nr].y_size+1)/2,
			      (max_y-img_list->img[img_nr].y_size)/2,
			      padding_value);
	    }     
	}
    }

  return(1);
}


int main (int argc, char **argv)
{
  struct arguments arguments;
  IMG_list* img_list;

  
  /* Set argument defaults */
  arguments.args[0] = "";
  arguments.args[1] = "";
  arguments.padding_value=0.0;
  arguments.mode=0;
  arguments.border=0;

  argp_parse (&argp, argc, argv, 0, 0, &arguments);
  
  img_list=IMG_load_list (arguments.args[0]);

  img_pad_list(img_list,arguments.padding_value,arguments.mode);
  IMG_add_border_list (img_list, arguments.border, arguments.border,
		       arguments.border, arguments.border,
		       arguments.padding_value);

  IMG_save_list(img_list,arguments.args[1]);

 exit(1);
}
